@@ -725,34 +725,32 @@ namespace netgen
   
   int Ngx_Mesh :: GetParentElement (int ei) const
   {
-      ei++;
-      if (mesh->GetDimension() == 3)
+    if (mesh->GetDimension() == 3)
       {
-          if (ei <= mesh->mlparentelement.Size())
-              return mesh->mlparentelement.Get(ei)-1;
+        if (ei < mesh->mlparentelement.Size())
+          return mesh->mlparentelement[ei];
       }
-      else
+    else
       {
-          if (ei <= mesh->mlparentsurfaceelement.Size())
-              return mesh->mlparentsurfaceelement.Get(ei)-1;
+        if (ei < mesh->mlparentsurfaceelement.Size())
+          return mesh->mlparentsurfaceelement[ei];
       }
-      return -1;
+    return -1;
   }
 
 
   int Ngx_Mesh :: GetParentSElement (int ei) const
   {
-      ei++;
-      if (mesh->GetDimension() == 3)
+    if (mesh->GetDimension() == 3)
       {
-          if (ei <= mesh->mlparentsurfaceelement.Size())
-              return mesh->mlparentsurfaceelement.Get(ei)-1;
+        if (ei < mesh->mlparentsurfaceelement.Size())
+          return mesh->mlparentsurfaceelement[ei];
       }
-      else
+    else
       {
-          return -1;
+        return -1;
       }
-      return -1;
+    return -1;
   }
 
   int Ngx_Mesh :: GetNIdentifications () const
@@ -1041,7 +1039,7 @@ namespace netgen
                                                         build_searchtree);
               return ind - 1;
             }
-          catch(NgException e) // quads not implemented curved yet
+          catch(const NgException & e) // quads not implemented curved yet
             {
           for (SegmentIndex si = 0; si < mesh->GetNSeg(); si++)
             {
@@ -1234,7 +1232,7 @@ namespace netgen
   {
     NgLock meshlock (mesh->MajorMutex(), true);
     Refinement & ref = const_cast<Refinement&> (mesh->GetGeometry()->GetRefinement());
-    ::netgen::HPRefinement (*mesh, &ref, SPLIT_ALFELD, 1, 0.5, true, true);
+    ::netgen::HPRefinement (*mesh, &ref, SPLIT_ALFELD, 1, 1.0/3.0, true, true);
   }
 
   
@@ -1249,9 +1247,15 @@ int Ngx_Mesh::GetElementOrder (int enr) const
 void Ngx_Mesh::GetElementOrders (int enr, int * ox, int * oy, int * oz) const
 {
   if (mesh->GetDimension() == 3)
-    mesh->VolumeElement(enr).GetOrder(*ox, *oy, *oz);
+    {
+      ElementIndex ei = IndexBASE<ElementIndex>() + enr-1;
+      mesh->VolumeElement(ei).GetOrder(*ox, *oy, *oz);
+    }
   else
-    mesh->SurfaceElement(enr).GetOrder(*ox, *oy, *oz);
+    {
+      SurfaceElementIndex sei = IndexBASE<SurfaceElementIndex>() + enr-1;      
+      mesh->SurfaceElement(sei).GetOrder(*ox, *oy, *oz);
+    }
 }
 
 void Ngx_Mesh::SetElementOrder (int enr, int order)
@@ -1297,6 +1301,36 @@ int Ngx_Mesh::GetClusterRepElement (int pi) const
 }
 
 
+int Ngx_Mesh::GetElement_Faces (int elnr, int * faces, int * orient) const
+{
+  const MeshTopology & topology = mesh->GetTopology();
+  if (mesh->GetDimension() == 3)
+    {
+      int num = topology.GetElementFaces (elnr+1, faces, orient);
+      for (int i = 0; i < num; i++)
+        faces[i]--;
+      return num;
+    }
+  else
+    {
+      faces[0] = elnr;
+      if (orient) orient[0] = 0;
+      return 1;
+    }
+}
+
+int Ngx_Mesh::GetSurfaceElement_Face (int selnr, int * orient) const
+{
+  if (mesh->GetDimension() == 3)
+    {
+      const MeshTopology & topology = mesh->GetTopology();
+      if (orient)
+	*orient = topology.GetSurfaceElementFaceOrientation (selnr+1);
+      return topology.GetFace (SurfaceElementIndex(selnr));
+    }
+  return -1;
+}
+
 
 
 //HERBERT: falsche Anzahl von Argumenten
