@@ -10,167 +10,64 @@
 #include <geometry2d.hpp>
 #include <meshing.hpp>
 
+#include "writeuser.hpp"
+#include "../general/gzstream.h"
+
+
 namespace netgen
 {
-#include "writeuser.hpp"
+  extern MeshingParameters mparam;
 
+  Array<UserFormatRegister::UserFormatEntry> UserFormatRegister::entries;
+  std::map<string, int> UserFormatRegister::format_to_entry_index;
 
-  void RegisterUserFormats (Array<const char*> & names,
-			    Array<const char*> & extensions)
+  void RegisterUserFormats (NgArray<const char*> & names,
+			    NgArray<const char*> & extensions)
 			    
 {
-  const char *types[] =
+  for (const auto & entry : UserFormatRegister::entries)
     {
-      "Neutral Format",  ".mesh",
-      "Surface Mesh Format", ".mesh" ,
-      "DIFFPACK Format", ".mesh",
-      "TecPlot Format", ".mesh",
-      "Tochnog Format", ".mesh",
-      "Abaqus Format", ".mesh",
-      "Fluent Format", ".mesh",
-      "Permas Format", ".mesh",
-      "FEAP Format", ".mesh",
-      "Elmer Format", "*",
-      "STL Format", ".stl",
-      "STL Extended Format", ".stl",
-      "VRML Format", ".*",
-      "Gmsh Format", ".gmsh",
-      "Gmsh2 Format", ".gmsh2",
-      "OpenFOAM 1.5+ Format", "*",
-	  "OpenFOAM 1.5+ Compressed", "*",
-      "JCMwave Format", ".jcm",
-      "TET Format", ".tet",
-      //      { "Chemnitz Format" },
-      0
-    };
-  
-  for (int i = 0; types[2*i]; i++)
-    {
-      names.Append (types[2*i]);
-      extensions.Append (types[2*i+1]);
+      names.Append (entry.format.c_str());
+      extensions.Append (entry.extensions[0].c_str());
     }
 }
   
-
-
 bool WriteUserFormat (const string & format,
 		      const Mesh & mesh,
-		      const NetgenGeometry & hgeom,
-		      const string & filename)
+		      const filesystem::path & filename)
 {
-  const CSGeometry & geom = *dynamic_cast<const CSGeometry*> (&hgeom);
+  if(!UserFormatRegister::HaveFormat(format))
+    return true;
 
-  PrintMessage (1, "Export mesh to file ", filename,
-		", format is ", format);
+  const auto entry = UserFormatRegister::Get(format);
+  if(!entry.write)
+    return true;
 
-  if (format == "Neutral Format")
-    WriteNeutralFormat (mesh, geom, filename);
-
-  else if (format == "Surface Mesh Format")
-    WriteSurfaceFormat (mesh, filename);
-
-  else if (format == "DIFFPACK Format")
-    WriteDiffPackFormat (mesh, geom, filename);
-
-  else if (format == "Tochnog Format")
-    WriteTochnogFormat (mesh, filename);
-
-  else if (format == "TecPlot Format")
-    cerr << "ERROR: TecPlot format currently out of order" << endl;
-      // WriteTecPlotFormat (mesh, geom, filename);
-
-  else if (format == "Abaqus Format")
-    WriteAbaqusFormat (mesh, filename);
-
-  else if (format == "Fluent Format")
-    WriteFluentFormat (mesh, filename);
-
-  else if (format == "Permas Format")
-    WritePermasFormat (mesh, filename);
-
-  else if (format == "FEAP Format")
-    WriteFEAPFormat (mesh, filename);
-
-  else if (format == "Elmer Format")
-    WriteElmerFormat (mesh, filename);
-
-  else if (format == "STL Format")
-    WriteSTLFormat (mesh, filename);
-
-  // Philippose - 16 August 2010
-  // Added additional STL Export in which
-  // each face of the geometry is treated
-  // as a separate "solid" entity
-  else if (format == "STL Extended Format")
-	WriteSTLExtFormat (mesh, filename);
-
-  else if (format == "VRML Format")
-    WriteVRMLFormat (mesh, 1, filename);
-
-  else if (format == "Fepp Format")
-    WriteFEPPFormat (mesh, geom, filename);
-
-  else if (format ==  "EdgeElement Format")
-    WriteEdgeElementFormat (mesh, geom, filename);
-
-  else if (format == "Chemnitz Format")
-    WriteUserChemnitz (mesh, filename);
-
-  else if (format == "Gmsh Format")
-    WriteGmshFormat (mesh, geom, filename);
-
-  // Philippose - 29/01/2009
-  // Added Gmsh v2.xx Mesh export capability
-  else if (format == "Gmsh2 Format")
-    WriteGmsh2Format (mesh, geom, filename);
-
-  // Philippose - 25/10/2009
-  // Added OpenFOAM 1.5+ Mesh export capability
-  else if (format == "OpenFOAM 1.5+ Format")
-    WriteOpenFOAM15xFormat (mesh, filename, false);
-
-  else if (format == "OpenFOAM 1.5+ Compressed")
-    WriteOpenFOAM15xFormat (mesh, filename, true);
-
-  else if (format == "JCMwave Format")
-    WriteJCMFormat (mesh, geom, filename);
-
-#ifdef OLIVER
-  else if (format == "TET Format")
-    WriteTETFormat( mesh, filename);//, "High Frequency" );
-#endif
-
-  else
-    {
-      return 1;
-    }
-
-  return 0;
+  (*entry.write)(mesh, filename);
+  return false;
 }
 
 
 
-
 /*
  *  Neutral mesh format
  *  points, elements, surface elements
  */
 
 void WriteNeutralFormat (const Mesh & mesh,
-			 const CSGeometry & geom,
-			 const string & filename)
+			 const filesystem::path & filename)
 {
   cout << "write neutral, new" << endl;
   int np = mesh.GetNP();
   int ne = mesh.GetNE();
   int nse = mesh.GetNSE();
   int nseg = mesh.GetNSeg();
-  int i, j;
+  // int i, j;
 
   int inverttets = mparam.inverttets;
   int invertsurf = mparam.inverttrigs;
 
-  ofstream outfile (filename.c_str());
+  ofstream outfile (filename);
 
   outfile.precision(6);
   outfile.setf (ios::fixed, ios::floatfield);
@@ -178,7 +75,7 @@ void WriteNeutralFormat (const Mesh & mesh,
 
   outfile << np << "\n";
 
-  for (i = 1; i <= np; i++)
+  for (int i = 1; i <= np; i++)
     {
       const Point3d & p = mesh.Point(i);
 
@@ -197,14 +94,18 @@ void WriteNeutralFormat (const Mesh & mesh,
   if (mesh.GetDimension() == 3)
     {
       outfile << ne << "\n";
-      for (i = 1; i <= ne; i++)
+      /*
+      for (int i = 1; i <= ne; i++)
 	{
 	  Element el = mesh.VolumeElement(i);
+      */
+      for (Element el : mesh.VolumeElements())
+        {
 	  if (inverttets)
 	    el.Invert();
 	  outfile.width(4);
 	  outfile << el.GetIndex() << "  ";
-	  for (j = 1; j <= el.GetNP(); j++)
+	  for (int j = 1; j <= el.GetNP(); j++)
 	    {
 	      outfile << " ";
 	      outfile.width(8);
@@ -215,14 +116,18 @@ void WriteNeutralFormat (const Mesh & mesh,
     }
 
   outfile << nse << "\n";
-  for (i = 1; i <= nse; i++)
+  /*
+  for (int i = 1; i <= nse; i++)
     {
       Element2d el = mesh.SurfaceElement(i);
+  */
+  for (Element2d el : mesh.SurfaceElements())
+    {
       if (invertsurf)
 	el.Invert();
       outfile.width(4);
       outfile << mesh.GetFaceDescriptor (el.GetIndex()).BCProperty() << "    ";
-      for (j = 1; j <= el.GetNP(); j++)
+      for (int j = 1; j <= el.GetNP(); j++)
 	{
 	  outfile << " ";
 	  outfile.width(8);
@@ -235,19 +140,31 @@ void WriteNeutralFormat (const Mesh & mesh,
   if (mesh.GetDimension() == 2)
     {
       outfile << nseg << "\n";
-      for (i = 1; i <= nseg; i++)
+      for (int i = 1; i <= nseg; i++)
 	{
 	  const Segment & seg = mesh.LineSegment(i);
 	  outfile.width(4);
 	  outfile << seg.si << "    ";
 
+          for (int j = 0; j < seg.GetNP(); j++)
+            {
+              outfile << " ";
+              outfile.width(8);
+              outfile << seg[j];
+            }
+          /*
 	  outfile << " ";
 	  outfile.width(8);
 	  outfile << seg[0];
 	  outfile << " ";
 	  outfile.width(8);
 	  outfile << seg[1];
-
+          if (seg[2] != -1)
+            {
+              outfile.width(8);
+              outfile << seg[2];
+            }
+          */
 	  outfile << "\n";
 	}
     }
@@ -262,14 +179,14 @@ void WriteNeutralFormat (const Mesh & mesh,
 
 
 void WriteSurfaceFormat (const Mesh & mesh,
-			 const string & filename)
+			 const filesystem::path & filename)
 {
   // surface mesh
   int i, j;
 
   cout << "Write Surface Mesh" << endl;
 
-  ofstream outfile (filename.c_str());
+  ofstream outfile (filename);
 
   outfile << "surfacemesh" << endl;
 
@@ -304,16 +221,17 @@ void WriteSurfaceFormat (const Mesh & mesh,
  */
 
 void WriteSTLFormat (const Mesh & mesh,
-		     const string & filename)
+		     const filesystem::path & filename)
 {
   cout << "\nWrite STL Surface Mesh" << endl;
 
+  auto ext = filename.extension();
   ostream *outfile;
 
-  if(filename.substr(filename.length()-3,3) == ".gz")
-	  outfile = new ogzstream(filename.c_str());
+  if(ext == ".gz")
+	  outfile = new ogzstream(filename);
   else
-	  outfile = new ofstream(filename.c_str());
+	  outfile = new ofstream(filename);
 
   int i;
 
@@ -361,22 +279,23 @@ void WriteSTLFormat (const Mesh & mesh,
  *    when using a third-party mesher
  */
 void WriteSTLExtFormat (const Mesh & mesh,
-		     const string & filename)
+		     const filesystem::path & filename)
 {
   cout << "\nWrite STL Surface Mesh (with separated boundary faces)" << endl;
 
+  auto ext = filename.extension();
   ostream *outfile;
 
-  if(filename.substr(filename.length()-3,3) == ".gz")
-	  outfile = new ogzstream(filename.c_str());
+  if(ext == ".gz")
+	  outfile = new ogzstream(filename);
   else
-	  outfile = new ofstream(filename.c_str());
+	  outfile = new ofstream(filename);
 
   outfile->precision(10);
 
   int numBCs = 0;
 
-  Array<int> faceBCs;
+  NgArray<int> faceBCs;
   TABLE<int> faceBCMapping;
 
   faceBCs.SetSize(mesh.GetNFD());
@@ -411,15 +330,15 @@ void WriteSTLExtFormat (const Mesh & mesh,
 
       for(int faceNr = 1;faceNr <= faceBCMapping.EntrySize(bcInd); faceNr++)
       {
-          Array<SurfaceElementIndex> faceSei;
+        Array<SurfaceElementIndex> faceSei;
           mesh.GetSurfaceElementsOfFace(faceBCMapping.Get(bcInd,faceNr),faceSei);
 
           for (int i = 0; i < faceSei.Size(); i++)
           {
         	  *outfile << "facet normal ";
-        	  const Point3d& p1 = mesh.Point(mesh.SurfaceElement(faceSei[i]).PNum(1));
-        	  const Point3d& p2 = mesh.Point(mesh.SurfaceElement(faceSei[i]).PNum(2));
-        	  const Point3d& p3 = mesh.Point(mesh.SurfaceElement(faceSei[i]).PNum(3));
+        	  const Point3d& p1 = mesh.Point(mesh[faceSei[i]].PNum(1));
+        	  const Point3d& p2 = mesh.Point(mesh[faceSei[i]].PNum(2));
+        	  const Point3d& p3 = mesh.Point(mesh[faceSei[i]].PNum(3));
 
         	  Vec3d normal = Cross(p2-p1,p3-p1);
         	  if (normal.Length() != 0)
@@ -453,7 +372,7 @@ void WriteSTLExtFormat (const Mesh & mesh,
 
 void WriteVRMLFormat (const Mesh & mesh,
 		      bool faces,
-		      const string & filename)
+		      const filesystem::path & filename)
 {
 
   if (faces)
@@ -466,7 +385,7 @@ void WriteVRMLFormat (const Mesh & mesh,
       int nse = mesh.GetNSE();
       int i, j;
 
-      ofstream outfile (filename.c_str());
+      ofstream outfile (filename);
 
       outfile.precision(6);
       outfile.setf (ios::fixed, ios::floatfield);
@@ -542,7 +461,7 @@ void WriteVRMLFormat (const Mesh & mesh,
       int nse = mesh.GetNSE();
       int i, j;
 
-      ofstream outfile (filename.c_str());
+      ofstream outfile (filename);
 
       outfile.precision(6);
       outfile.setf (ios::fixed, ios::floatfield);
@@ -608,6 +527,16 @@ void WriteVRMLFormat (const Mesh & mesh,
 
 }
 
+void WriteVRMLFormatLineset (const Mesh & mesh, const filesystem::path & filename)
+{
+  WriteVRMLFormat(mesh, false, filename);
+}
+
+void WriteVRMLFormatFaceset (const Mesh & mesh, const filesystem::path & filename)
+{
+  WriteVRMLFormat(mesh, true, filename);
+}
+
 
 
 
@@ -617,11 +546,10 @@ void WriteVRMLFormat (const Mesh & mesh,
  * FEPP .. a finite element package developed at University Linz, Austria
  */
 void WriteFEPPFormat (const Mesh & mesh,
-		      const CSGeometry & geom,
-		      const string & filename)
+		      const filesystem::path & filename)
 {
 
-  ofstream outfile (filename.c_str());
+  ofstream outfile (filename);
 
   if (mesh.GetDimension() == 3)
 
@@ -632,7 +560,7 @@ void WriteFEPPFormat (const Mesh & mesh,
       int np = mesh.GetNP();
       int ne = mesh.GetNE();
       int nse = mesh.GetNSE();
-      int ns = mesh.GetNFD();
+      // int ns = mesh.GetNFD();
       int i, j;
 
       outfile.precision(5);
@@ -663,9 +591,13 @@ void WriteFEPPFormat (const Mesh & mesh,
 
 
       outfile << ne << "\n";
+      /*
       for (i = 1; i <= ne; i++)
 	{
 	  const Element & el = mesh.VolumeElement(i);
+      */
+      for (const Element & el : mesh.VolumeElements())
+        {
 	  outfile.width(4);
 	  outfile << el.GetIndex() << " ";
 	  outfile.width(4);
@@ -708,7 +640,7 @@ void WriteFEPPFormat (const Mesh & mesh,
 	}
       */
 
-
+      /*
       // write CSG surfaces
       if (&geom && geom.GetNSurf() >= ns)
 	{
@@ -717,6 +649,7 @@ void WriteFEPPFormat (const Mesh & mesh,
 	    geom.GetSurface(mesh.GetFaceDescriptor(i).SurfNr())->Print(outfile);
 	}
       else
+      */
 	outfile << "0" << endl;
     }
 
@@ -747,8 +680,7 @@ void WriteFEPPFormat (const Mesh & mesh,
  */
 
 void WriteEdgeElementFormat (const Mesh & mesh,
-			     const CSGeometry & geom,
-			     const string & filename)
+			     const filesystem::path & filename)
 {
   cout << "write edge element format" << endl;
 
@@ -757,13 +689,12 @@ void WriteEdgeElementFormat (const Mesh & mesh,
   int nelements = mesh.GetNE();
   int nsurfelem = mesh.GetNSE();
   int nedges = top->GetNEdges();
-  int i, j;
 
   int inverttets = mparam.inverttets;
   int invertsurf = mparam.inverttrigs;
-  Array<int> edges;
+  NgArray<int> edges;
 
-  ofstream outfile (filename.c_str());
+  ofstream outfile (filename);
 
   outfile.precision(6);
   outfile.setf (ios::fixed, ios::floatfield);
@@ -772,7 +703,7 @@ void WriteEdgeElementFormat (const Mesh & mesh,
 
   // vertices with coordinates
   outfile << npoints << "\n";
-  for (i = 1; i <= npoints; i++)
+  for (int i = 1; i <= npoints; i++)
     {
       const Point3d & p = mesh.Point(i);
 
@@ -786,38 +717,47 @@ void WriteEdgeElementFormat (const Mesh & mesh,
 
   // element - edge - list
   outfile << nelements << " " << nedges << "\n";
+  /*
   for (i = 1; i <= nelements; i++)
     {
       Element el = mesh.VolumeElement(i);
+  */
+  for (ElementIndex ei : Range(mesh.VolumeElements()))
+    {
+      int i = ei-IndexBASE(ei)+1;
+      
+      Element el = mesh.VolumeElement(ei);
+
       if (inverttets)
       	el.Invert();
       outfile.width(4);
       outfile << el.GetIndex() << "  ";
       outfile.width(8);
       outfile << el.GetNP();
-      for (j = 1; j <= el.GetNP(); j++)
+      for (int j = 1; j <= el.GetNP(); j++)
 	{
 	  outfile << " ";
 	  outfile.width(8);
 	  outfile << el.PNum(j);
 	}
 
-      top->GetElementEdges(i,edges);
+      // top->GetElementEdges(i,edges);
+      auto eledges = top->GetEdges(ei);
       outfile << endl << "      ";
       outfile.width(8);
-      outfile << edges.Size();
-      for (j=1; j <= edges.Size(); j++)
+      outfile << eledges.Size();
+      for (int j=1; j <= eledges.Size(); j++)
 	{
 	  outfile << " ";
 	  outfile.width(8);
-	  outfile << edges[j-1];
+	  outfile << eledges[j-1]+1;
 	}
       outfile << "\n";
 
       // orientation:
       top->GetElementEdgeOrientations(i,edges);
       outfile << "              ";
-      for (j=1; j <= edges.Size(); j++)
+      for (int j=1; j <= edges.Size(); j++)
 	{
 	  outfile << " ";
 	  outfile.width(8);
@@ -828,42 +768,45 @@ void WriteEdgeElementFormat (const Mesh & mesh,
 
   // surface element - edge - list (with boundary conditions)
   outfile << nsurfelem << "\n";
-  for (i = 1; i <= nsurfelem; i++)
+  for (int i = 1; i <= nsurfelem; i++)
     {
-      Element2d el = mesh.SurfaceElement(i);
+      SurfaceElementIndex sei(i-1);
+      Element2d el = mesh[sei];
       if (invertsurf)
 	el.Invert();
       outfile.width(4);
       outfile << mesh.GetFaceDescriptor (el.GetIndex()).BCProperty() << "  ";
       outfile.width(8);
       outfile << el.GetNP();
-      for (j = 1; j <= el.GetNP(); j++)
+      for (int j = 1; j <= el.GetNP(); j++)
 	{
 	  outfile << " ";
 	  outfile.width(8);
 	  outfile << el.PNum(j);
 	}
 
-      top->GetSurfaceElementEdges(i,edges);
+      // top->GetSurfaceElementEdges(i,edges);
+      auto edges = top->GetEdges(sei);
       outfile << endl << "      ";
       outfile.width(8);
       outfile << edges.Size();
-      for (j=1; j <= edges.Size(); j++)
+      for (int j=0; j < edges.Size(); j++)
 	{
 	  outfile << " ";
 	  outfile.width(8);
-	  outfile << edges[j-1];
+	  outfile << edges[j]+1;
 	}
       outfile << "\n";
     }
 
 
-  int v1, v2;
+  // int v1, v2;
   // edge - vertex - list
   outfile << nedges << "\n";
-  for (i=1; i <= nedges; i++)
+  for (int i=1; i <= nedges; i++)
     {
-      top->GetEdgeVertices(i,v1,v2);
+      // top->GetEdgeVertices(i,v1,v2);
+      auto [v1,v2] = top->GetEdgeVertices(i-1);
       outfile.width(4);
       outfile << v1;
       outfile << " ";
@@ -872,169 +815,14 @@ void WriteEdgeElementFormat (const Mesh & mesh,
     }
 }
 
-
-
-
-
-
-
-
-
-#ifdef OLDSTYLE_WRITE
-
-
-void WriteFile (int typ,
-		const Mesh & mesh,
-		const CSGeometry & geom,
-		const char * filename,
-		const char * geomfile,
-		double h)
-{
-
-
-  int inverttets = mparam.inverttets;
-  int invertsurf = mparam.inverttrigs;
-
-
-
-
-
-
-
-
-  if (typ == WRITE_EDGEELEMENT)
-    {
-      // write edge element file
-      // Peter Harscher, ETHZ
-
-      cout << "Write Edge-Element Format" << endl;
-
-      ofstream outfile (filename);
-
-      int i, j;
-      int ned;
-
-      // hash table representing edges;
-      INDEX_2_HASHTABLE<int> edgeht(mesh.GetNP());
-
-      // list of edges
-      Array<INDEX_2> edgelist;
-
-      // edge (point) on boundary ?
-      BitArray bedge, bpoint(mesh.GetNP());
-
-      static int eledges[6][2] = { { 1, 2 } , { 1, 3 } , { 1, 4 },
-				   { 2, 3 } , { 2, 4 } , { 3, 4 } };
-
-      // fill hashtable   (point1, point2)  ---->  edgenr
-      for (i = 1; i <= mesh.GetNE(); i++)
-	{
-	  const Element & el = mesh.VolumeElement (i);
-	  INDEX_2 edge;
-	  for (j = 1; j <= 6; j++)
-	    {
-	      edge.I1() = el.PNum (eledges[j-1][0]);
-	      edge.I2() = el.PNum (eledges[j-1][1]);
-	      edge.Sort();
-
-	      if (!edgeht.Used (edge))
-		{
-		  edgelist.Append (edge);
-		  edgeht.Set (edge, edgelist.Size());
-		}
-	    }
-	}
-
-
-      // set bedges, bpoints
-      bedge.SetSize (edgelist.Size());
-      bedge.Clear();
-      bpoint.Clear();
-
-      for (i = 1; i <= mesh.GetNSE(); i++)
-	{
-	  const Element2d & sel = mesh.SurfaceElement(i);
-	  for (j = 1; j <= 3; j++)
-	    {
-	      bpoint.Set (sel.PNum(j));
-
-	      INDEX_2 edge;
-	      edge.I1() = sel.PNum(j);
-	      edge.I2() = sel.PNum(j%3+1);
-	      edge.Sort();
-
-	      bedge.Set (edgeht.Get (edge));
-	    }
-	}
-
-
-
-      outfile << mesh.GetNE() << endl;
-      // write element ---> point
-      for (i = 1; i <= mesh.GetNE(); i++)
-	{
-	  const Element & el = mesh.VolumeElement(i);
-
-	  outfile.width(8);
-	  outfile << i;
-	  for (j = 1; j <= 4; j++)
-	    {
-	      outfile.width(8);
-	      outfile << el.PNum(j);
-	    }
-	  outfile << endl;
-	}
-
-      // write element ---> edge
-      for (i = 1; i <= mesh.GetNE(); i++)
-	{
-	  const Element & el = mesh.VolumeElement (i);
-	  INDEX_2 edge;
-	  for (j = 1; j <= 6; j++)
-	    {
-	      edge.I1() = el.PNum (eledges[j-1][0]);
-	      edge.I2() = el.PNum (eledges[j-1][1]);
-	      edge.Sort();
-
-	      outfile.width(8);
-	      outfile << edgeht.Get (edge);
-	    }
-	  outfile << endl;
-	}
-
-      // write points
-      outfile << mesh.GetNP() << endl;
-      outfile.precision (6);
-      for (i = 1; i <= mesh.GetNP(); i++)
-	{
-	  const Point3d & p = mesh.Point(i);
-
-	  for (j = 1; j <= 3; j++)
-	    {
-	      outfile.width(8);
-	      outfile << p.X(j);
-	    }
-	  outfile << "       "
-		  << (bpoint.Test(i) ? "1" : 0) << endl;
-	}
-
-      // write edges
-      outfile << edgelist.Size() << endl;
-      for (i = 1; i <= edgelist.Size(); i++)
-	{
-	  outfile.width(8);
-	  outfile << edgelist.Get(i).I1();
-	  outfile.width(8);
-	  outfile << edgelist.Get(i).I2();
-	  outfile << "       "
-		  << (bedge.Test(i) ? "1" : "0") << endl;
-	}
-    }
-
-
-
+static RegisterUserFormat reg_neutral("Neutral Format", {".mesh"}, ReadFile, WriteNeutralFormat);
+static RegisterUserFormat reg_surface("Surface Mesh Format", {".mesh", ".surf"} ,ReadFile, WriteSurfaceFormat);
+static RegisterUserFormat reg_stl ("STL Format", {".stl", ".stlb"}, ReadFile, WriteSTLFormat);
+static RegisterUserFormat reg_stlx ("STL Extended Format", {".stl", ".stlb"}, nullopt, WriteSTLExtFormat);
+static RegisterUserFormat reg_vrml ("VRML Format", {".*"}, nullopt, WriteVRMLFormatLineset);
+static RegisterUserFormat reg_vrml_faces ("VRML Format Faces", {".*"}, nullopt, WriteVRMLFormatFaceset);
+static RegisterUserFormat reg_fepp ("Fepp Format", {"*"}, nullopt, WriteFEPPFormat);
+static RegisterUserFormat reg_edgeelement ("EdgeElement Format", {"*"}, nullopt, WriteEdgeElementFormat);
 
-}
-#endif
 }
 
