@@ -13,7 +13,7 @@ FrontPoint3 :: FrontPoint3 ()
   globalindex.Invalidate(); //  = -1;
   nfacetopoint = 0; 
   frontnr = 1000; 
-  cluster = 0;
+  cluster = PointIndex::INVALID;
 }
 
 
@@ -23,7 +23,7 @@ FrontPoint3 :: FrontPoint3 (const Point<3> & ap, PointIndex agi)
   globalindex = agi;
   nfacetopoint = 0; 
   frontnr = 1000; 
-  cluster = 0;
+  cluster = PointIndex::INVALID;
 }
 
 
@@ -35,7 +35,7 @@ FrontFace :: FrontFace ()
   qualclass = 1; 
   oldfront = 0; 
   hashvalue = 0;
-  cluster = 0;
+  cluster = PointIndex::INVALID;
 }
 
 FrontFace :: FrontFace (const MiniElement2d & af)
@@ -71,7 +71,7 @@ AdFront3 :: AdFront3 ()
     hashtable.Init(&points, &faces);
 
   facetree = NULL;
-  connectedpairs = NULL;
+  // connectedpairs = NULL;
 
   rebuildcounter = -1;
   lasti = 0;
@@ -82,7 +82,7 @@ AdFront3 :: AdFront3 ()
 AdFront3 :: ~AdFront3 ()
 {
   delete facetree;
-  delete connectedpairs;
+  // delete connectedpairs;
 }
 
 void AdFront3 :: GetPoints (NgArray<Point<3> > & apoints) const
@@ -153,10 +153,10 @@ INDEX AdFront3 :: AddFace (const MiniElement2d & aface)
     }
 
 
-  int cluster = 0;
+  PointIndex cluster = PointIndex::INVALID;
   for (i = 1; i <= aface.GetNP(); i++)
     {
-      if (points[aface.PNum(i)].cluster)
+      if (points[aface.PNum(i)].cluster.IsValid())
 	cluster = points[aface.PNum(i)].cluster;
     }
   for (i = 1; i <= aface.GetNP(); i++)
@@ -182,9 +182,13 @@ void AdFront3 :: DeleteFace (INDEX fi)
 {
   nff--;
 
+  /*
   for (int i = 1; i <= faces.Get(fi).Face().GetNP(); i++)
     {
       PointIndex pi = faces.Get(fi).Face().PNum(i);
+  */
+  for (PointIndex pi : faces.Get(fi).Face().PNums())
+    {
       points[pi].RemoveFace();
       if (!points[pi].Valid())
 	delpointl.Append (pi);
@@ -213,13 +217,13 @@ void AdFront3 :: DeleteFace (INDEX fi)
 }
 
 
-INDEX AdFront3 :: AddConnectedPair (const INDEX_2 & apair)
+INDEX AdFront3 :: AddConnectedPair (PointIndices<2> apair)
 {
   if (!connectedpairs)
-    connectedpairs = new TABLE<int, PointIndex::BASE> (GetNP());
+    connectedpairs = make_unique<DynamicTable<PointIndex, PointIndex>> (GetNP());
 
-  connectedpairs->Add (apair.I1(), apair.I2());
-  connectedpairs->Add (apair.I2(), apair.I1());
+  connectedpairs->Add (apair[0], apair[1]);
+  connectedpairs->Add (apair[1], apair[0]);
 
   return 0;
 }
@@ -231,11 +235,11 @@ void AdFront3 :: CreateTrees ()
   PointIndex pi;
   Point3d pmin, pmax;
 
-  for (pi = PointIndex::BASE; 
-       pi < GetNP()+PointIndex::BASE; pi++)
+  for (pi = IndexBASE<PointIndex>(); 
+       pi < GetNP()+IndexBASE<PointIndex>(); pi++)
     {
       const Point<3> & p = GetPoint(pi);
-      if (pi == PointIndex::BASE)
+      if (pi == IndexBASE<PointIndex>())
 	{
 	  pmin = p;
 	  pmax = p;
@@ -323,12 +327,12 @@ void AdFront3 :: RebuildInternalTables ()
 	{
 	  const MiniElement2d & el = faces.Get(i).Face();
 
-	  int mini = points[el.PNum(1)].cluster;
-	  int maxi = mini;
+	  PointIndex mini = points[el.PNum(1)].cluster;
+	  PointIndex maxi = mini;
 	  
 	  for (int j = 2; j <= 3; j++)
 	    {
-	      int ci = points[el.PNum(j)].cluster;
+	      PointIndex ci = points[el.PNum(j)].cluster;
 	      if (ci < mini) mini = ci;
 	      if (ci > maxi) maxi = ci;
 	    }
@@ -366,7 +370,7 @@ void AdFront3 :: RebuildInternalTables ()
       cntcl++;
   */
   
-  NgArray<double, PointIndex::BASE> clvol (np);
+  Array<double, PointIndex> clvol (np);
   clvol = 0.0;
 
   for (int i = 1; i <= faces.Size(); i++)
@@ -397,21 +401,18 @@ void AdFront3 :: RebuildInternalTables ()
 
 
 
-  int negvol = 0;
-  for (int i = PointIndex::BASE; 
-       i < clvol.Size()+PointIndex::BASE; i++)
-    {
-      if (clvol[i] < 0)
-	negvol = 1;
-    }
+  bool negvol = false;
+  for (auto i : clvol.Range())
+    if (clvol[i] < 0)
+      negvol = true;
   
   if (negvol)
     {
       for (int i = 1; i <= faces.Size(); i++)
-	faces.Elem(i).cluster = 1;
+	faces.Elem(i).cluster = IndexBASE<PointIndex>();
       // for (PointIndex pi = points.Begin(); pi < points.End(); pi++)
       for (PointIndex pi : points.Range())
-	points[pi].cluster = 1;
+	points[pi].cluster = IndexBASE<PointIndex>();
     }
 
   if (hashon) 
@@ -424,8 +425,6 @@ void AdFront3 :: RebuildInternalTables ()
 
 int AdFront3 :: SelectBaseElement ()
 {
-  int i, hi, fstind;
-
   /*
   static int minval = -1;
   static int lasti = 0;
@@ -450,12 +449,12 @@ int AdFront3 :: SelectBaseElement ()
     }
     */
   
-  fstind = 0;
+  int fstind = 0;
 
-  for (i = lasti+1; i <= faces.Size() && !fstind; i++)
+  for (int i = lasti+1; i <= faces.Size() && !fstind; i++)
     if (faces.Elem(i).Valid())
       {
-	hi = faces.Get(i).QualClass() +
+	int hi = faces.Get(i).QualClass() +
 	  points[faces.Get(i).Face().PNum(1)].FrontNr() +
 	  points[faces.Get(i).Face().PNum(2)].FrontNr() +
 	  points[faces.Get(i).Face().PNum(3)].FrontNr();
@@ -471,10 +470,10 @@ int AdFront3 :: SelectBaseElement ()
   if (!fstind)
     {
       minval = INT_MAX;
-      for (i = 1; i <= faces.Size(); i++)
+      for (int i = 1; i <= faces.Size(); i++)
 	if (faces.Elem(i).Valid())
 	  {
-	    hi = faces.Get(i).QualClass() +
+	    int hi = faces.Get(i).QualClass() +
 	      points[faces.Get(i).Face().PNum(1)].FrontNr() +
 	      points[faces.Get(i).Face().PNum(2)].FrontNr() +
 	      points[faces.Get(i).Face().PNum(3)].FrontNr();
@@ -495,10 +494,10 @@ int AdFront3 :: SelectBaseElement ()
 
 
 int AdFront3 :: GetLocals (int fstind,
-			   NgArray<Point3d, PointIndex::BASE> & locpoints,
-			   NgArray<MiniElement2d> & locfaces,   // local index
-			   NgArray<PointIndex, PointIndex::BASE> & pindex,
-			   NgArray<INDEX> & findex,
+			   Array<Point3d, PointIndex> & locpoints,
+			   Array<MiniElement2d> & locfaces,   // local index
+			   Array<PointIndex, PointIndex> & pindex,
+			   Array<INDEX> & findex,
 			   INDEX_2_HASHTABLE<int> & getconnectedpairs,
 			   float xh,
 			   float relh,
@@ -515,7 +514,7 @@ int AdFront3 :: GetLocals (int fstind,
       hashcreated=1;
     }
 
-  INDEX i, j;
+  INDEX i;
   PointIndex pstind;
   Point3d midp, p0;
 
@@ -529,7 +528,7 @@ int AdFront3 :: GetLocals (int fstind,
   locfaces3.SetSize(0);
   findex2.SetSize(0);
 
-  int cluster = faces.Get(fstind).cluster;
+  PointIndex cluster = faces.Get(fstind).cluster;
 
   pstind = faces.Get(fstind).Face().PNum(1);
   p0 = points[pstind].P();
@@ -596,28 +595,37 @@ int AdFront3 :: GetLocals (int fstind,
 
 
   invpindex.SetSize (points.Size());
+  /*
   for (i = 1; i <= locfaces.Size(); i++)
     for (j = 1; j <= locfaces.Get(i).GetNP(); j++)
       {
 	PointIndex pi = locfaces.Get(i).PNum(j);
         invpindex[pi] = PointIndex::INVALID;
       }
+  */
+  for (auto & f : locfaces)
+    for (int j = 1; j <= f.GetNP(); j++)
+      {
+	PointIndex pi = f.PNum(j);
+        invpindex[pi] = PointIndex::INVALID;
+      }
 
-  for (i = 1; i <= locfaces.Size(); i++)
+  // for (i = 1; i <= locfaces.Size(); i++)
+  for (auto & f : locfaces)
     {
-      for (j = 1; j <= locfaces.Get(i).GetNP(); j++)
+      // for (j = 1; j <= locfaces.Get(i).GetNP(); j++)
+      for (int j = 1; j <= f.GetNP(); j++)
 	{
-	  PointIndex pi = locfaces.Get(i).PNum(j);
+	  // PointIndex pi = locfaces.Get(i).PNum(j);
+          PointIndex pi = f.PNum(j);
 	  if (!invpindex[pi].IsValid())
 	    {
 	      pindex.Append (pi);
               locpoints.Append (points[pi].P());
-	      invpindex[pi] = pindex.Size()-1+PointIndex::BASE;
+	      invpindex[pi] = pindex.Size()-1+IndexBASE<PointIndex>();
             }
-          // locfaces.Elem(i).PNum(j) = locpoints.Append (points[pi].P());
-          // }
-	  // else
-          locfaces.Elem(i).PNum(j) = invpindex[pi];
+          // locfaces.Elem(i).PNum(j) = invpindex[pi];
+          f.PNum(j) = invpindex[pi];          
 	}
     }
 
@@ -625,22 +633,27 @@ int AdFront3 :: GetLocals (int fstind,
 
   if (connectedpairs)
     {
-      for (i = 1; i <= locpoints.Size(); i++)
+      // for (i = 1; i <= locpoints.Size(); i++)
+      for (auto i : locpoints.Range())
 	{
-	  int pind = pindex.Get(i);
-	  if (pind >= 1 && pind <= connectedpairs->Size ())
+	  PointIndex pind = pindex[i]; // .Get(i);
+	  // if (pind.IsValid() && pind <= connectedpairs->Size ())
+          if (connectedpairs->Range().Contains(pind))
 	    {
-	      for (j = 1; j <= connectedpairs->EntrySize(pind); j++)
+	      // for (int j = 1; j <= connectedpairs->EntrySize(pind); j++)
+              for (auto j : (*connectedpairs)[pind].Range())
 		{
-		  int oi = connectedpairs->Get(pind, j);
-		  int other = invpindex.Get(oi);
-		  if (other >= 1 && other <= pindex.Size() && 
-		      pindex.Get(other) == oi)
+		  //PointIndex oi = connectedpairs->Get(pind, j);
+                  PointIndex oi = (*connectedpairs)[pind][j];
+		  PointIndex other = invpindex[oi];
+		  // if (other >= 1 && other <= pindex.Size() &&
+                  if (pindex.Range().Contains(other) &&
+		      pindex[other] == oi)
 		    {
 		      // INDEX_2 coned(i, other);
 		      // coned.Sort();
 		      // (*testout) << "connected: " << locpoints.Get(i) << "-" << locpoints.Get(other) << endl;
-		      getconnectedpairs.Set (INDEX_2::Sort (i, other), 1);
+		      getconnectedpairs.Set (PointIndices<2>::Sort (i, other), 1);
 		    }
 		}
 	    }
@@ -667,10 +680,10 @@ int AdFront3 :: GetLocals (int fstind,
 
 // returns all points connected with fi
 void AdFront3 :: GetGroup (int fi,
-			   NgArray<MeshPoint, PointIndex::BASE> & grouppoints,
-			   NgArray<MiniElement2d> & groupelements,
-			   NgArray<PointIndex, PointIndex::BASE> & pindex,
-			   NgArray<INDEX> & findex) 
+			   Array<MeshPoint, PointIndex> & grouppoints,
+			   Array<MiniElement2d> & groupelements,
+			   Array<PointIndex, PointIndex> & pindex,
+			   Array<INDEX> & findex) 
 {
   // static NgArray<char> pingroup;
   int changed;
@@ -734,8 +747,9 @@ void AdFront3 :: GetGroup (int fi,
     if (points[pi].Valid())
       {
 	grouppoints.Append (points[pi].P());
-	pindex.Append (pi);
-	invpindex[pi] = pindex.Size();
+        pindex.Append (pi);
+        // invpindex[pi] = pindex.Size();
+	invpindex[pi] = pindex.Size()-1 + IndexBASE<PointIndex>();
       }
 
   for (int i = 1; i <= faces.Size(); i++)
@@ -787,8 +801,8 @@ void AdFront3 :: SetStartFront (int /* baseelnp */)
     */
 }
 
-bool AdFront3 :: PointInsideGroup(const NgArray<PointIndex, PointIndex::BASE> &grouppindex,
-                                  const NgArray<MiniElement2d> &groupfaces) const
+bool AdFront3 :: PointInsideGroup(const Array<PointIndex, PointIndex> &grouppindex,
+                                  const Array<MiniElement2d> &groupfaces) const
 {
   for(auto pi : Range(points))
     {
@@ -797,7 +811,7 @@ bool AdFront3 :: PointInsideGroup(const NgArray<PointIndex, PointIndex::BASE> &g
       for(const auto& f : groupfaces)
         {
           for(auto i : Range(3))
-            if(grouppindex.Get(f.PNum(i+1)) == pi)
+            if(grouppindex[f.PNum(i+1)] == pi)
               {
                 found = true;
                 break;
@@ -814,9 +828,9 @@ bool AdFront3 :: PointInsideGroup(const NgArray<PointIndex, PointIndex::BASE> &g
       int count = 0;
       for(const auto& f : groupfaces)
         {
-          const auto& p1 = points[grouppindex.Get(f.PNum(1))].P();
-          auto v1 = points[grouppindex.Get(f.PNum(2))].P() - p1;
-          auto v2 = points[grouppindex.Get(f.PNum(3))].P() - p1;
+          const auto& p1 = points[grouppindex[f.PNum(1)]].P();
+          auto v1 = points[grouppindex[f.PNum(2)]].P() - p1;
+          auto v2 = points[grouppindex[f.PNum(3)]].P() - p1;
           for(auto i : Range(3))
             {
               a(i,0) = v1[i];
