@@ -8,6 +8,7 @@
 
 #include <mystdlib.h>
 #include "meshing.hpp"
+#include "boundarylayer.hpp"
 // #include <csg.hpp>
 // #include <geometry2d.hpp>
 #include <../interface/rw_medit.hpp>
@@ -598,7 +599,11 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
           py::format_descriptor<int[3]>::format(),
           py::detail::npy_format_descriptor<int[3]>::dtype() },
         py::detail::field_descriptor {
-          "index", offsetof(Segment, edgenr), sizeof(int),
+          "index", offsetof(Segment, si), sizeof(int),
+          py::format_descriptor<int>::format(),
+          py::detail::npy_format_descriptor<int>::dtype() },
+        py::detail::field_descriptor {
+          "edgenr", offsetof(Segment, edgenr), sizeof(int),
           py::format_descriptor<int>::format(),
           py::detail::npy_format_descriptor<int>::dtype() },
       });
@@ -960,11 +965,13 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
            );
       })
     .def_property_readonly("parentelements", [](Mesh & self) {
-      return FlatArray<int>(self.mlparentelement.Size(), &self.mlparentelement[0]);
+      // return FlatArray<int>(self.mlparentelement.Size(), &self.mlparentelement[0]);
+      return FlatArray(self.mlparentelement);
     }, py::keep_alive<0,1>())
     .def_property_readonly("parentsurfaceelements", [](Mesh & self) {
-      return FlatArray<int>(self.mlparentsurfaceelement.Size(),
-                            &self.mlparentsurfaceelement[0]);
+      // return FlatArray<int>(self.mlparentsurfaceelement.Size(),
+      // &self.mlparentsurfaceelement[0]);
+      return FlatArray(self.mlparentsurfaceelement);      
     }, py::keep_alive<0,1>())
     .def_property_readonly("macromesh", [](Mesh & self) {
       auto coarsemesh = make_shared<Mesh>();
@@ -987,7 +994,11 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
     .def("FaceDescriptor", static_cast<FaceDescriptor&(Mesh::*)(int)> (&Mesh::GetFaceDescriptor),
          py::return_value_policy::reference)
     .def("GetNFaceDescriptors", &Mesh::GetNFD)
-    
+    .def("RestrictLocalH", [](Mesh& self, const Point<3>& pnt, double maxh,
+                              int layer)
+    {
+      self.RestrictLocalH(pnt, maxh, layer);
+    }, py::arg("p"), py::arg("h"), py::arg("layer")=1)
     .def("FaceDescriptors", 
          // static_cast<Array<Element>&(Mesh::*)()> (&Mesh::FaceDescriptors),
          &Mesh::FaceDescriptors,         
@@ -1233,7 +1244,7 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
             else
               throw Exception("either 'dim' or 'codim' must be specified");
             
-            NgArray<string*> & codimnames = self.GetRegionNamesCD (codim);
+            Array<string*> & codimnames = self.GetRegionNamesCD (codim);
             
             std::vector<string> names;
             for (auto name : codimnames)
@@ -1265,7 +1276,10 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
            py::list points;
            for(const auto& pair : self.GetIdentifications().GetIdentifiedPoints())
              {
-               py::tuple pnts = py::make_tuple(pair.first.I1(), pair.first.I2());
+               // py::tuple pnts = py::make_tuple(pair.first.I1(), pair.first.I2());
+               
+               auto [pi1, pi2] = get<0> (pair.first);
+               py::tuple pnts = py::make_tuple(pi1, pi2);
                points.append(pnts);
              }
            return points;
@@ -1471,13 +1485,14 @@ py::arg("point_tolerance") = -1.)
     .def ("BoundaryLayer2", GenerateBoundaryLayer2, py::arg("domain"), py::arg("thicknesses"), py::arg("make_new_domain")=true, py::arg("boundaries")=Array<int>{})
     .def ("BoundaryLayer", [](Mesh & self, variant<string, int, std::vector<int>> boundary,
                               variant<double, std::vector<double>> thickness,
-                              variant<string, map<string, string>> material,
+                              optional<variant<string, map<string, string>>> material,
                               variant<string, int, std::vector<int>> domain, bool outside,
                               optional<variant<string, std::vector<int>>> project_boundaries,
                               bool grow_edges, bool limit_growth_vectors,
                               bool sides_keep_surfaceindex,
-                              bool keep_surfaceindex)
+                              bool disable_curving)
            {
+             throw Exception("Call syntax has changed! Pass a list of BoundaryLayerParameters to the GenerateMesh call instead: \ngeo.GenerateMesh(..., boundary_layers=[BoundaryLayerParameters(...), BoundaryLayerParameters(...), ...])");
              BoundaryLayerParameters blp;
              blp.boundary = boundary;
              blp.thickness = thickness;
@@ -1488,13 +1503,13 @@ py::arg("point_tolerance") = -1.)
              blp.grow_edges = grow_edges;
              blp.limit_growth_vectors = limit_growth_vectors;
              blp.sides_keep_surfaceindex = sides_keep_surfaceindex;
-             blp.keep_surfaceindex = keep_surfaceindex;
+             blp.disable_curving = disable_curving;
              GenerateBoundaryLayer (self, blp);
              self.UpdateTopology();
-           }, py::arg("boundary"), py::arg("thickness"), py::arg("material"),
+           }, py::arg("boundary"), py::arg("thickness"), py::arg("material")=nullopt,
           py::arg("domains") = ".*", py::arg("outside") = false,
-          py::arg("project_boundaries")=nullopt, py::arg("grow_edges")=true, py::arg("limit_growth_vectors") = true, py::arg("sides_keep_surfaceindex")=false,
-          py::arg("keep_surfaceindex")=false, "Add boundary layer to mesh. see help(BoundaryLayerParameters) for details.")
+          py::arg("project_boundaries")=nullopt, py::arg("grow_edges")=true, py::arg("limit_growth_vectors") = false, py::arg("sides_keep_surfaceindex")=false,
+          py::arg("disable_curving")=true, "Add boundary layer to mesh. see help(BoundaryLayerParameters) for details.")
 
     .def_static ("EnableTableClass", [] (string name, bool set)
           {
@@ -1560,7 +1575,7 @@ py::arg("point_tolerance") = -1.)
                 {
                     const auto & seg = segs[i];
                     for(auto k : Range(2))
-                        output[2*i+k] = seg[k]-PointIndex::BASE;
+                      output[2*i+k] = seg[k]-IndexBASE<PointIndex>();
                 } });
             return output;
           })
@@ -1579,8 +1594,8 @@ py::arg("point_tolerance") = -1.)
                   // PointIndex p0,p1;
                   // topo.GetEdgeVertices(i+1, p0, p1);
                   auto [p0,p1] = topo.GetEdgeVertices(i);
-                    output[2*i] = p0-PointIndex::BASE;
-                    output[2*i+1] = p1-PointIndex::BASE;
+                    output[2*i] = p0-IndexBASE<PointIndex>();
+                    output[2*i+1] = p1-IndexBASE<PointIndex>();
                 } });
             return output;
           })
@@ -1598,7 +1613,7 @@ py::arg("point_tolerance") = -1.)
                     const auto & sel = surfels[i];
                     auto * trig = &trigs[3*i];
                     for(auto k : Range(3))
-                        trig[k] = sel[k]-PointIndex::BASE;
+                        trig[k] = sel[k]-IndexBASE<PointIndex>();
                         // todo: quads (store the second trig in thread-local extra array, merge them at the end (mutex)
                 } });
             return trigs;
@@ -1616,7 +1631,7 @@ py::arg("point_tolerance") = -1.)
                     const auto & el = els[i];
                     auto * trig = &tets[4*i];
                     for(auto k : Range(4))
-                        trig[k] = el[k]-PointIndex::BASE;
+                        trig[k] = el[k]-IndexBASE<PointIndex>();
                         // todo: prisms etc (store the extra tets in thread-local extra array, merge them at the end (mutex)
                 } });
             return tets;
@@ -1724,15 +1739,14 @@ py::arg("point_tolerance") = -1.)
     .def(py::init([]( 
         std::variant<string, int, std::vector<int>> boundary,
         std::variant<double, std::vector<double>> thickness,
-        std::variant<string, std::map<string, string>> new_material,
+        std::optional<std::variant<string, std::map<string, string>>> new_material,
         std::variant<string, int, std::vector<int>> domain,
         bool outside,
         std::optional<std::variant<string, std::vector<int>>> project_boundaries,
         bool grow_edges,
         bool limit_growth_vectors,
-        bool sides_keep_surfaceindex,
-        bool keep_surfaceindex,
-        double limit_safety)
+        std::optional<bool> sides_keep_surfaceindex,
+        bool disable_curving)
         {
           BoundaryLayerParameters blp;
           blp.boundary = boundary;
@@ -1744,15 +1758,14 @@ py::arg("point_tolerance") = -1.)
           blp.grow_edges = grow_edges;
           blp.limit_growth_vectors = limit_growth_vectors;
           blp.sides_keep_surfaceindex = sides_keep_surfaceindex;
-          blp.keep_surfaceindex = keep_surfaceindex;
-          blp.limit_safety = limit_safety;
+          blp.disable_curving = disable_curving;
           return blp;
         }),
-           py::arg("boundary"), py::arg("thickness"), py::arg("new_material"),
+           py::arg("boundary"), py::arg("thickness"), py::arg("new_material")=nullopt,
            py::arg("domain") = ".*", py::arg("outside") = false,
            py::arg("project_boundaries")=nullopt, py::arg("grow_edges")=true,
-           py::arg("limit_growth_vectors") = true, py::arg("sides_keep_surfaceindex")=false,
-           py::arg("keep_surfaceindex")=false, py::arg("limit_safety")=0.3,
+           py::arg("limit_growth_vectors") = false, py::arg("sides_keep_surfaceindex")=nullopt,
+           py::arg("disable_curving")=true,
            R"delimiter(
 Add boundary layer to mesh.
 
@@ -1804,8 +1817,7 @@ project_boundaries : Optional[str] = None
     .def_readwrite("grow_edges", &BoundaryLayerParameters::grow_edges)
     .def_readwrite("limit_growth_vectors", &BoundaryLayerParameters::limit_growth_vectors)
     .def_readwrite("sides_keep_surfaceindex", &BoundaryLayerParameters::sides_keep_surfaceindex)
-    .def_readwrite("keep_surfaceindex", &BoundaryLayerParameters::keep_surfaceindex)
-    .def_readwrite("limit_safety", &BoundaryLayerParameters::limit_safety)
+    .def_readwrite("disable_curving", &BoundaryLayerParameters::disable_curving)
     ;
   py::implicitly_convertible<py::dict, BoundaryLayerParameters>();
 
