@@ -190,7 +190,7 @@ proc AddRecentMeshFile { filename } {
     }
 loadmeshinifile;
 
-# astrid ende
+# astrid end
 
 
 .ngmenu.file add command -label "Save Mesh..." -accelerator "<s><m>" \
@@ -222,20 +222,28 @@ loadmeshinifile;
 
 
 
+set meshimportformats [Ng_GetImportFormats]
 
 .ngmenu.file add command -label "Import Mesh..." \
     -command { 
+	foreach importformat $meshimportformats {
+	    if { [lindex $importformat 0] == $importfiletype } {
+		set extension [lindex $importformat 1]
+	    }
+	}
 	set types {
 	    {"Neutral format"  {.mesh .emt} }
 	    {"Surface mesh format"  {.surf} }
 	    {"Universal format"  {.unv} }
 	    {"Olaf format"  {.emt} }
 	    {"TET format" {.tet} }
+	    {"STL format" {.stl .stlb} }
 	    {"Pro/ENGINEER neutral format" {.fnf} }
+	    {"CFD General Notation System" {.cgns} }
 	          }
-	set file [tk_getOpenFile -filetypes $types ]
+  set file [tk_getOpenFile -filetypes $meshimportformats -typevariable importfiletype]
 	if {$file != ""} {
-	    Ng_ImportMesh $file 
+	    Ng_ImportMesh $file $importfiletype
 	    set selectvisual mesh
 	    Ng_SetVisParameters
 	    redraw
@@ -244,6 +252,8 @@ loadmeshinifile;
     }
 
 
+set meshexportformats [Ng_GetExportFormats]
+
 .ngmenu.file add command -label "Export Mesh..." \
     -command {
 
@@ -261,8 +271,11 @@ loadmeshinifile;
         } elseif { $exportfiletype == "OpenFOAM 1.5+ Compressed"} {
 	    set file [file nativename [tk_chooseDirectory -title "OpenFOAM 1.5+ Mesh Export - Select Case Directory"]]
         } else {
-#	    set file [tk_getSaveFile  -filetypes "{ \"$exportfiletype\" {$extension} }" ]
-	    set file [tk_getSaveFile  -filetypes "{ \"$exportfiletype\" {*}}" ]
+            # set file [tk_getSaveFile  -filetypes "{ \"$exportfiletype\" {$extension} }" ]
+	    # set file [tk_getSaveFile  -filetypes "{ \"$exportfiletype\" {*}}" ]
+            set file [tk_getSaveFile  -filetypes $meshexportformats -typevariable exportfiletype]
+            puts "type = $exportfiletype"
+            puts "filename = $file"
 	}
 
 	if {$file != ""} {
@@ -271,9 +284,12 @@ loadmeshinifile;
     }
 
 .ngmenu.file add cascade -label "Export Filetype" -menu .ngmenu.file.filetype 
-
 menu .ngmenu.file.filetype 
 
+foreach exportformat $meshexportformats {
+    .ngmenu.file.filetype add radio -label [lindex $exportformat 0] -variable exportfiletype -command { .ngmenu.file invoke "Export Mesh..." }
+}
+
 
 .ngmenu.file add separator
 
@@ -1122,7 +1138,7 @@ proc timer2 { } {
 	}
 
     }
-    after 10 { timer2 }
+    after 40 { timer2 }
 }
 # after 1000 { timer2 }
 timer2
